#!/usr/bin/perl
package POE::Component::goanseech::seriesConnector;
##############################################################################
=pod

=head1 NAME

POE::Component::goanseech::seriesConnector

=head1 DESCRIPTION

extracts the itneresteing links from a given HTML file and
recusrively parses the files that are pointed inside if they match something
inside the patternServer.


=head1 AUTHOR

  Bruno Bttcher <bboett at adlp dot org>

=head1 COPYRIGHT and LICENCE

 Copyright (c) 2002 Bruno Boettcher

 patternQuery.pl is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; version 2
 of the License.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


head1 Methods of this class 

=over 

=cut

##############################################################################
use warnings;
use strict;
use Data::Dumper;

use POE;
use POE::Filter::Reference;
#use POE::Component::Client::TCP;

##############################################################################
=pod

=item connectWithServer

prepare the connection with the patternServer, set up all callback to be ready
as soon as the POE system is started

=cut

######################################################################
sub connectWithServer
{
  my ($this,$hrefs,$kernel) = @_[OBJECT,ARG0, KERNEL];
  push(@{$this->{"jobs"}}, $hrefs);
  #POE::Component::Client::TCP->new
  if(scalar(@{$this->{"jobs"}}))
  {
    if($this->{"connected"})
    {
      #print("connection buildup casting sendBatch  to $this->{name}\n");
      $kernel->post($this->{"name"}, "sendBatch" );
      return;
    }# if($this->{"connected"})

    #print("connecting with server\n");
    use ClientTcp;
    my $clientSocket = 
      ClientTcp->new
      ( 
       Alias => $this->{"tcpalias"},
       RemoteAddress => $this->{"settings"}->{"host"},
       RemotePort    => $this->{"settings"}->{"port"},
       Filter        => "POE::Filter::Reference",
       SSL => 1,

       # Build a request and send it.

       Connected => sub {
	 my($kernel,$heap) = @_[KERNEL,HEAP];
	 $this->connected($kernel,$heap);
       },

       # Receive a response, display it, and shut down the client.

       ServerInput => sub {
	 my($kernel,$heap,$arg) = @_[KERNEL,HEAP,ARG0];
	 $this->got_response($kernel,$heap,$arg);
       },
       sendBatch => sub {
	 my($kernel,$heap,$arg) = @_[KERNEL,HEAP,ARG0];
	 $this->sendBatch($kernel,$heap,$arg);
       },
       #could reach the server....
       ConnectError => sub {
	 my($kernel,$heap,$arg) = @_[KERNEL,HEAP,ARG0];
	 $this->disconnected($kernel,$heap,$arg);
       },
       Disconnected => sub {
	 my($kernel,$heap,$arg) = @_[KERNEL,HEAP,ARG0];
	 $this->disconnected($kernel,$heap,$arg);
       },
       );
  }# if(scalar(@{$this->{"jobs"}}))
  else
  {
    print("no queries to submit to the server...\n");
    #bail out...
    print("bailing out to ".$this->{"defaultSession"}." at ".$this->{"defaultEvent"}."\n");
    $kernel->post($this->{"defaultSession"}, $this->{"defaultEvent"},$this->{"torrents"}); 
  }# else

}#sub connectWithServer
##############################################################################
=pod

=item connected

we are now connected with the server, login and cast an event if there is
allready some data outstanding

=cut
######################################################################
sub connected
{
  my ($this,$kernel,$heap) = @_;
  $this->{"connected"} = 1;
  $this->{"server"} = $heap->{server};

  #print("connected!! obj is ".ref($this)."\n");

  my @request = ( 
      "auth ".$this->{"settings"}->{"user"}." ".$this->{"settings"}->{"passwd"},
      );

    my $job = {
      "requests"	=> \@request,
      "session"		=> $this->{"name"},
      "msgevent"	=> "gotMsg",
    };
  unshift(@{$this->{"jobs"}},$job);

  #print("shell initialized send out: ".Dumper(\@request)."\n");
  #$heap->{server}->put( \@request );
  #$_[KERNEL]->delay( sendBatch => 1 );
  #print("connection success casting sendBatch  to $this->{name}\n");
  $kernel->post($this->{"name"}, "sendBatch" ) if(!($this->{"actjob"}));
  #print("connected! posted sendBatch\n");
}#sub connected
##############################################################################
=pod

=item got_response

the server send us a response, should be a pattern that matched somewhere...

=cut
######################################################################
sub got_response
{
  my ($this, $kernel, $heap, $hash ) = @_;

  #$kernel->post('delegate',"hello");

  if ( !$hash) 
  {
    #ignore empty lines...
    print("got empty input....!\n");
  }
  elsif ( ref($hash) =~ /nameContainer/ ) 
  {
    #print ("Client received a nameContainer type: ". ref($hash). " with $hash\n");
    #print ("dumping: ". Dumper($hash). "\n");
    #print "Client received:".Dumper($hash)."\n";
    #print ("Client received:".Dumper($hash)."\n");
    my $list = $hash->{"listing"};
    #print ("listing:".Dumper($list)."\n");
    #print ("got nameContainer for ".Dumper($this->{"actjob"})."\n");
  }
  elsif ( ref($hash) eq 'HASH' ) 
  {
    push(@{$this->{"actjob"}->{"result"}}, $hash);
  }
  elsif ( ref($hash) eq 'SCALAR' ) 
  {
    #print "Client received:".$$hash."\n";
    if($$hash =~ /quit/i)
    {
      #print("close of the client asked\n");
      $kernel->post($this->{"name"}, "shutdown"); 
      #exit(0);
    }# if($$hash =~ /quit/i)
    elsif($$hash =~ /jobDone/i)
    {
      $this->{"jobcount"}--;
      #print("jobDone actual count :$this->{jobcount}\n");
      if($this->{"jobcount"} <= 0)
      {
	$this->{"jobcount"} = 0; 
	#$kernel->yield('quit');
	#$kernel->yield("shutdown"); 
	#print("all jobs done! check if we still have URLs to parse:".Dumper($this->{"url_list"})."\n");
	my $actJob = $this->{"actjob"};

	if($actJob)
	{
	  #my $msg = "jodone sending back data";
	  #$msg .=" to ".$actJob->{"session"} if($actJob->{"session"});
	  #$msg .= " with ".$actJob->{"event"} if($actJob->{"event"});
	  #$msg .= "\n";
	  #print($msg);
	  if($actJob->{"session"} && $actJob->{"event"} && $actJob->{"result"})
	  {
	    $kernel->post($actJob->{"session"}, $actJob->{"event"}, $actJob->{"result"}); 
	  }# if($actJob->{"session"} && $actJob->{"event"} && $actJob->{"result"})
	  #print("jobdone deleting ".Dumper($this->{"actjob"})."\n");
	  delete $this->{"actjob"};
	}# if($actJob)
	$kernel->post($this->{"name"}, "sendBatch"); 
      }#if($this->{"jobcount"} <= 0)
    }# elsif($$hash =~ /jobDone/i)
    elsif($$hash =~ /redirectprinting/i)
    {
      my($cmd,$mode) = split(/ /,$$hash,2);
      #print("Seriesconnector got redirect printing request mode $mode\n");

      if($mode =~ /on/i)
      {
	$this->{"redirectPrint"} = 1;
      }# if($this->{"redirectPrint"})
      else
      {
	delete($this->{"redirectPrint"});
      }# else
    }#elsif($$hash =~ /redirectprinting/i)
    else
    {
      #print("don't know how to handle $$hash\n for actJob:".Dumper($this->{"actjob"})."\n");
      #print("handling scalar data '".substr($$hash,0,30)."' for actJob:".Dumper($this->{"actjob"})."\n");
	my $actJob = $this->{"actjob"};
      if($actJob->{"session"} && $actJob->{"msgevent"})
      {
	#print("posting data ".substr($$hash,0,30)."\nback to ".$actJob->{"msgevent"}."@".$actJob->{"session"}."\n");
	$kernel->post($actJob->{"session"}, $actJob->{"msgevent"}, $hash); 
      }# if($actJob->{"session"} && $actJob->{"event"} && $actJob->{"result"})
      else
      {
	$kernel->post($this->{"defaultSession"}, $this->{"msgevent"}, $hash); 
      }# if($actJob->{"session"} && $actJob->{"event"} && $actJob->{"result"})
    }# else
  }# elsif ( ref($hash) eq 'SCALAR' ) 
  else 
  {
    #print "Client received an unknown response type: ", ref($hash), " with $hash\n";
    print ("Client received an unknown response type: ". ref($hash). " with $hash\n");
  }
  #$kernel->yield("shutdown");
}#sub got_response
##############################################################################
=pod

=item sendBatch

send out the whole list of requests

=cut
######################################################################
sub sendBatch
{
  my ($this, $kernel, $hash ) = @_[OBJECT,KERNEL,ARG0];
  #print("sendBatch sending the stuff \n");
  if($this->{"actjob"})
  {
    #print("watcha calling me? still running ".$this->{"actjob"}->{"event"}."\n");
    print("watcha calling me? still running ".Dumper($this->{"actjob"})."\n");
    if($this->{"actjob"}->{"tmout"} && time() > $this->{"actjob"}->{"tmout"})
    {
     print("job ".Dumper($this->{"actjob"})." tmouted\n"); 
     my $actJob = $this->{"actjob"};
	    $kernel->post($actJob->{"session"}, $actJob->{"event"}, $actJob->{"result"}); 
     delete($this->{"actjob"});
    }# if($this->{"actjob"}->{"tmout"} && time() > $this->{"actjob"}->{"tmout"})
    #just to be sure, recall this one a bit later
    my $delay = $this->{"settings"}->{"batchdelay"} * 60;
    $_[KERNEL]->delay( sendBatch => $delay, $hash );
    return 0;
  }# if($this->{"actjob"})

  $this->{"url_list"} = [];

  my $nextJob = shift(@{$this->{"jobs"}});
  if($nextJob)
  {
    #print("seriesConnector next job: ".Dumper($nextJob));
    my $requests = $nextJob->{"requests"};
    #forget oneshot requests
    $this->{"actjob"} = $nextJob if($nextJob->{"session"});


    while(@$requests)
    {
      my @request;
      for(my $i = 0; ($i < 20) && @$requests; $i++)
      {
	push(@request, shift(@$requests)); 
      }#for(my $i = 0; ($i < 20) && @$requests; $i++)
      $this->{"jobcount"}++ if($nextJob->{"session"});
      #print("sendBatch posting job".Dumper(\@request)."\n");
      $this->{"server"}->put( \@request );
      @request = ();
    }# while(@$requests)
  }# if($nextJob)
  else
  {
    #print("sendBatch: no more jobs\n");
    #print("reporting back to ".$this->{"defaultSession"}." at ".$this->{"defaultEvent"}."\n");
    $_[KERNEL]->post($this->{"defaultSession"}, $this->{"defaultEvent"},$this->{"torrents"}); 
  }# else
  return 1;
}# sub sendBatch
##############################################################################
=pod

=item startSession

start our own session to be able to have timered stuff

=cut
######################################################################
sub startSession
{
  my ($this) = @_;
  POE::Session->create
    (
     object_states =>
     [
     $this => { 
     _start         => "_start",
     _stop         => "_stop",
     "shutdown" => "shutdown",
     sendBatch => "sendBatch",
     "doneParsing" => "connectWithServer",
     "gotMsg" => "gotMsg",
     "print" => "print_ev",
     }
  ]
    );
}# sub startSession
##############################################################################
=pod

=item _start

to keep this session alive set an alias

=cut
######################################################################
sub _start
{
  my ($this, $kernel, $arg ) = @_[OBJECT, KERNEL, ARG0 ];
  $_[KERNEL]->alias_set($this->{"name"});
}# sub _start
##############################################################################
=pod

=item _stop

to keep this session alive set an alias

=cut
######################################################################
sub _stop
{
  my ($this, $kernel, $arg ) = @_[OBJECT, KERNEL, ARG0 ];
  print("stopping series connector...\n");
  #exit(0);
}# sub _stop
##############################################################################
=pod

=item shutdown

close this session

=cut
######################################################################
sub shutdown
{
  my ($this, $kernel, $arg ) = @_[OBJECT, KERNEL, ARG0 ];
  #print("shutdown extract torrents...\n");
  delete $this->{"server"};
  undef $this->{"server"};
  #shut down the ssl client
  $_[KERNEL]->post($this->{"tcpalias"}, "shutdown"); 
  #$kernel->yield("shutdown");
  #print("TCP client should now shutdown\n");
  $_[KERNEL]->alias_remove($this->{"name"} );
  print("shutdown seriesclient, making exit!...\n");
  exit();
}# sub shutdown
##############################################################################
=pod

=item init

Initiaize, means set up this module

=cut

######################################################################
sub init
{
  my ($this,$settings,$arguments) = @_;

  $this->{"settings"} = $settings;

  $this->{"jobcount"} = 0;
  #fetch the stuff from the file
  $this->{"torrents"} = []; 
  $this->{"visitedURL"} = {}; 
  $this->{"tcpalias"} = "tcpClient";


  foreach my $key (keys(%$arguments))
  {
    $this->{$key} = $arguments->{$key};
  }# foreach my $key (keys($arguments))

  $this->startSession();

}#sub init
##############################################################################
=pod

=item CTOR

instantiation of the Helper

=cut

##############################################################################
sub new 
{
  my($class,$sessionname) = @_;
  my $this = bless {}, "POE::Component::goanseech::seriesConnector";
  $this->{"name"} = "anon session".time;
  $this->{"name"} = $sessionname;
  return $this;
}
##############################################################################
=pod

=item disconnected

somehow we lost the connection...

=cut

##############################################################################
sub disconnected 
{
  my($this,$kernel,$heap,$arg) = @_;
  #print("lost connection with the server\n");
  $this->{"connected"} = 0;
  $kernel->post($this->{"tcpalias"}, "shutdown");
}
##############################################################################
=pod

=item gotMsg 

got something from the default signal....

=cut

##############################################################################
sub gotMsg
{
   my ( $kernel, $hash, $heap ) = @_[ KERNEL, ARG0, HEAP ];
   #print("POE::Component::goanseech::seriesConnector got spourious msg $$hash\n");
}# sub gotMsg
#########################################################
=pod

=item print

delegator for printing

=cut

#########################################################
sub print_ev
{
  my ($this,$msg,$kernel,$origin) = @_[OBJECT,ARG0, KERNEL,ARG1];

  $origin = "torrent" if(!$origin);
  if($this->{"redirectPrint"})
  {
    my $handler = $this->{"printhandler"};
    my @request = ( 
	[ "print", { "msg" =>$msg, "origin" => "torrent"} ]
	);
    #reset the listing of pages to seach, since we are asking for a whole new bunch of pages...
    my $job = {
      "requests" => \@request,
    };
    $kernel->yield("doneParsing", $job);
  }# if($this->{"printhandler"})
  else
  {
    print("$msg\n");
  }
}# sub print
1
__END__


=back

=head1 SEE ALSO

http://poe.perl.org/

=cut

